LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

package vga is
	component vgaSync is
		PORT (	clk25 : in std_logic;	--25.175 (or close enough!) pixel clock
					reset : in std_logic;	--master reset
					hires : in std_logic;	-- 1 if in 640x480 mode, 0 = 320x240					
					hSync : out std_logic;	--horizontal sync output
					vSync : out std_logic;	--verticle sync output
					hClock : out std_logic	--horizontal pixel clock (only active when pixels are to be clocked)
				);
	end component;
end vga;

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;

entity vgaSync is
	PORT (	clk25 : in std_logic;	--25.175 (or close enough!) pixel clock
				reset : in std_logic;	--master reset
				hires : in std_logic;	-- 1 if in 640x480 mode, 0 = 320x240
				hSync : out std_logic;	--horizontal sync output
				vSync : out std_logic;	--verticle sync output
				hClock : out std_logic	--horizontal pixel clock (only active when pixels are to be clocked)
			);
end vgaSync;

architecture impl1 of vgaSync is

signal hClockCnt: unsigned (9 downto 0);	-- 10 bits count

signal hSyncInternal: std_logic;	-- internal sync for fields (set on rizing edge, hSync is set on falling edge)

signal vClockCnt: unsigned (9 downto 0);	-- 10 bits for count
signal clk2: std_logic;
signal hRaster, vRaster: std_logic;

alias pixelClock is clk25;
--signal pixelClock: std_logic;

begin
	process(clk25)
	begin
		if(reset = '1') then
			clk2 <= '0';
		elsif(rising_edge(clk25)) then
			clk2 <= not clk2;
		end if;
	end process;

	--pixelClock <= (clk25 and hires) or (clk2 and not hires);
	
	process(pixelClock, reset, hClockCnt) 
	begin
		if(hClockCnt = 8) then 
			--front porch 8 pixels
			hSync <= '0';
			hSyncInternal <= '0';
		elsif(hClockCnt = (8+96)) then
			--sync 96 pixels
			hSync <= '1';
		end if;
	
		if(hClockCnt = (8+96+48)) then
			--back porch and left border 48 pixels
			hRaster <= '1';
		end if;
	
		if(hires = '1') then 
			if(hClockCnt = (8+96+48+640)) then
				-- raster
				hRaster <= '0';
			end if;
		
			if(hClockCnt = (8+96+48+640+6)) then
					hSyncInternal <= '1';
			end if;
		else 
			if(hClockCnt = (8+96+48+320)) then
				-- raster
				hRaster <= '0';
			end if;
		
			if(hClockCnt = (8+96+48+320+7)) then
					hSyncInternal <= '1';
			end if;
		end if;
		
		if(reset = '1') then
			hClockCnt <= (others => '0');
			hSync	<= '1';
			hRaster <= '0';
			hSyncInternal <= '0';
		elsif(rising_edge(pixelClock)) then
			if(hires = '1') then 
				if(hClockCnt = (8+96+48+640+7)) then
					--border 8 pixels
					hClockCnt <= (others => '0');
				else 
					hClockCnt <= hClockCnt+1;
				end if;
			else
				if(hClockCnt = (8+96+48+320+7)) then
					--border 8 pixels
					hClockCnt <= (others => '0');
				else 
					hClockCnt <= hClockCnt+1;
				end if;			
			end if;
		end if; 

		--output horizontal pixel clock ?
		if((hRaster = '1') and (vRaster = '1')) then
			hClock <= pixelClock;
		else 
			hClock <= '0';
		end if;		
	end process;

	--Vert Processing 
	process(hSyncInternal, reset)
	begin
		if(reset = '1') then
			vClockCnt <= (others => '0');
			vSync	<= '1';
			vRaster <= '0';
		elsif(rising_edge(hSyncInternal)) then
			if(vClockCnt = 2) then
				-- 2 lines front porch
				vSync <= '0';
			elsif(vClockCnt = (2+2)) then
				-- 2 lines vertical sync
				vSync <= '1';
			end if;
			if(vClockCnt = (2+2+25+8)) then
				-- 25 lines back porch + 8 lines top border
				vRaster <= '1';
			end if;
			if(hires = '1') then
				if(vClockCnt = (2+2+25+8+480)) then
					-- 480 lines video
					vRaster <= '0';
				end if;
				if(vClockCnt = (2+2+25+8+480+7)) then
					-- 8 lines bottom border
					vClockCnt <= (others => '0');
				else
					vClockCnt <= vClockCnt + 1;
				end if;
			else
				if(vClockCnt = (2+2+25+8+240)) then
					-- 480 lines video
					vRaster <= '0';
				end if;
				if(vClockCnt = (2+2+25+8+240+7)) then
					-- 8 lines bottom border
					vClockCnt <= (others => '0');
				else
					vClockCnt <= vClockCnt + 1;
				end if;
			end if;
		end if;	
	end process;
end ;

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;

entity vga12bpp is
	port (
		reset: in std_logic;
		pixelClk: in std_logic;
		r,g,b: out std_logic_vector(3 downto 0);
		ramAddr: out std_logic_vector(9 downto 0);
		ramData: in std_logic_vector(15 downto 0);
		ramReq: out std_logic;	-- request pixel data
		ramOk: in std_logic -- for ram timing
	);
end vga12bpp;

architecture impl1 of vga12bpp is
	signal pixelCnt, ramPtr: unsigned (9 downto 0);
	signal nextR, nextG, nextB: std_logic_vector(3 downto 0);

begin
	process(pixelClk)
	begin
		if(reset = '1') then
			pixelCnt <= (others => '0');
			ramPtr <= (others => '0');
		else
			if(rising_edge(pixelClk)) then
				case pixelCnt(1 downto 0) is
					when "00" =>
						r <= ramData(3 downto 0);
						g <= ramData(7 downto 4);
						b <= ramData(11 downto 8);
						nextR <= ramData(15 downto 12);
					when "01" =>
						r <= nextR;
						g <= ramData(3 downto 0);
						b <= ramData(7 downto 4);
						nextR <= ramData(11 downto 8);
						nextG <= ramData(15 downto 12);
					when "10" =>
						r <= nextR;
						g <= nextG;
						b <= ramData(3 downto 0);
						nextR <= ramData(7 downto 4);
						nextG <= ramData(11 downto 8);
						nextB <= ramData(15 downto 12);
					when "11" =>
						r <= nextR;
						g <= nextG;
						b <= nextB;
					when others =>
				end case;
			elsif(falling_edge(pixelClk)) then
				pixelCnt <= pixelCnt + 1;
				if not (pixelCnt(1 downto 0) = 3) then
					ramPtr <= ramPtr + 1;
				end if;
			end if;
		end if;
	end process;
	ramAddr <= std_logic_vector(ramPtr);
end;
